from tests.fixtures import decorator


@decorator(function=True)
def function(request):  # pragma: no cover
    return request


class Class(object):
    @decorator(method=True)
    def method(self, request):  # pragma: no cover
        return request


class Instance(object):
    def __call__(self, request):  # pragma: no cover
        return request


inst = Instance()
inst = decorator(instance=True)(inst)



from tests.fixtures import decorator

from tests.fixtures.importonly.two import twofunction  # should not be scanned


@decorator(function=True)
def onefunction(request):  # pragma: no cover
    return request



from tests.fixtures import decorator


class Super(object):  # pragma: no cover
    @decorator()
    def classname(self):
        pass

    @decorator()
    def boo(self):
        pass


# the Sub class must not inherit the decorations of its superclass when scanned


class Sub(Super):  # pragma: no cover
    pass



from venusian import lift
from tests.fixtures import decorator


class Super1(object):  # pragma: no cover
    @decorator()
    def classname(self):
        pass

    @decorator()
    def boo(self):
        pass

    @decorator()
    def ram(self):
        pass

    def jump(self):
        pass


class Super2(object):  # pragma: no cover
    def boo(self):
        pass

    @decorator()
    def hiss(self):
        pass

    @decorator()
    def jump(self):
        pass


@lift()
class Sub(Super1, Super2):  # pragma: no cover
    def boo(self):
        pass

    def hiss(self):
        pass

    @decorator()
    def smack(self):
        pass



import venusian


class decorator(object):
    category = None
    call_count = 0

    def __init__(self, **kw):
        self.__dict__.update(kw)

    def __call__(self, wrapped):
        view_config = self.__dict__.copy()

        def callback(context, name, ob):
            if hasattr(context, "test"):
                context.test(ob=ob, name=name, **view_config)
            self.__class__.call_count += 1

        info = venusian.attach(wrapped, callback, category=self.category)
        if info.scope == "class":
            # we're in the midst of a class statement
            if view_config.get("attr") is None:
                view_config["attr"] = wrapped.__name__
        return wrapped


class categorydecorator(decorator):
    category = "mycategory"


class categorydecorator2(decorator):
    category = "mycategory2"



from venusian import lift, onlyliftedfrom
from tests.fixtures import (
    categorydecorator,
    categorydecorator2,
)


@onlyliftedfrom()
class Super(object):  # pragma: no cover
    @categorydecorator()
    def hiss(self):
        pass

    @categorydecorator2()
    def jump(self):
        pass


@lift(("mycategory",))
class Sub(Super):  # pragma: no cover
    def hiss(self):
        pass

    def jump(self):
        pass

    @categorydecorator2()
    def smack(self):
        pass
